#include "CPolygon.h"
#include "CRockPoly.h"
#include <time.h>
#define TIME_STEP 1000
#define TANK_LENGTH 48.0
#define TANK_WIDTH 24.0
#define TANK_HEIGHT 24.0

#include "flameangel.h"
#include "chromis.h"
#include "bluetang.h"
#include "yellowtang.h"
#include "anthias.h"
#include "flasherwrasse.h"
#include "chrysurus.h"

bool drawSchoolLines = FALSE;

class CFishPoly : public CPolygon {
      private:

              VERTEX m_vOrigin;
              VERTEX m_vRotation;
              VERTEX m_vVelocity;
              VERTEX m_vLastPosition;
              VERTEX m_vLastVelocity;
              VERTEX m_vNextVelocity;
              float m_fRotation;
              float m_fMinRange;
              float m_fMinFishRange;
              float m_fMaxSchoolRange;
              char * m_dataFile;
              float m_fActivity;
              float m_fMaxSpeed;
              int m_iLastMove;
              int m_iCurrentMove;
              int m_iSteps;
              int m_iTotalSteps;
              int m_iTimeStep;
              int m_fishType;
              float m_Rank;
              VERTEX m_vSchoolCenter;
              VERTEX m_vSchoolVelocity;

              bool m_schools;
              bool m_schooled;

              CFishPoly * m_fish;
              int m_fishCount;

              CRockPoly * m_rocks;
              int m_rockCount;
              bool willhit;
      public:
             CFishPoly();
             ~CFishPoly() { delete[] m_dataFile, delete[] m_fish, delete[] m_rocks;}
             void init(float data[][7], int datasize);
             void init(float data[][7], int datasize, int texID);
             void draw();
             void move();
             void animate();
             void computeVelocity();
             void detectCollisions();
             void school();
             void loadBehavior(char * dataFile);
             void loadBehavior(float data[][7], int datasize);
             int getType();
             VERTEX getNextPosition();
             VERTEX getPosition();
             VERTEX getNextVelocity();
             VERTEX getVelocity();
             float getRank();


             void setSchoolVelocity(VERTEX v);
             void setSchoolCenter(VERTEX v);
             void setSchooled(bool b);
             void setAllFish(CFishPoly * fishes, int fishCount);
             void setAllRocks(CRockPoly * rocks, int rockCount);
             int hitsRock(CRockPoly * rock);
             void getBoundingBox(VERTEX pos, VERTEX vel, VERTEX* points);
};

CFishPoly::CFishPoly()
    :CPolygon()
{
   // outfile << "Entering CFishPoly()\n";
    srand ( GetTickCount() * rand() );



    float rnd, rndmax = RAND_MAX;

    rnd = rand();

    m_iTimeStep = (int)(TIME_STEP*((rnd/rndmax) + 0.5));
    //m_iTimeStep = TIME_STEP;

    rnd = rand();
    m_vOrigin.x = (TANK_LENGTH-6.0)*(rnd/rndmax) - (TANK_LENGTH-6.0)/2.0;
    rnd = rand();
    m_vOrigin.y = (TANK_WIDTH-6.0)*(rnd/rndmax) - (TANK_WIDTH-6.0)/2.0;
    rnd = rand();
    m_vOrigin.z = (TANK_HEIGHT-6.0)*(rnd/rndmax) - (TANK_HEIGHT-6.0)/2.0;


    m_vOrigin.x = 0.0;
    m_vOrigin.y = 0.0;
    m_vOrigin.z = 0.0;


    m_vVelocity.x = 0.0;
    m_vVelocity.y = 0.0;
    m_vVelocity.z = 0.0;
    m_vNextVelocity.x = 0.0;
    m_vNextVelocity.y = 0.0;
    m_vNextVelocity.z = 0.0;
    m_vLastVelocity.x = 0.0;
    m_vLastVelocity.y = 0.0;
    m_vLastVelocity.z = 0.0;

    m_vSchoolCenter.x = 0.0;
    m_vSchoolCenter.y = 0.0;
    m_vSchoolCenter.z = 0.0;
    m_vSchoolVelocity.x = 0.0;
    m_vSchoolVelocity.y = 0.0;
    m_vSchoolVelocity.z = 0.0;

    m_vRotation.x = 0.0;
    m_vRotation.y = 0.0;
    m_vRotation.z = 0.0;
    m_fRotation = 0.0;

    m_fMinRange = 2.5;

    m_fActivity = 0.0;
    m_fMaxSpeed = 0.0;

    m_iLastMove = 0;

    m_iSteps = 50;
    m_iTotalSteps= 50;

    m_schooled = FALSE;

    m_rockCount = 0;

    willhit = FALSE;

}

void CFishPoly::init(float data[][7], int datasize)
{
    m_fishType = (int)data[0][0];
    loadEllipse(data,datasize);
    loadBehavior(data,datasize);
}

void CFishPoly::init(float data[][7], int datasize, int texID)
{
    m_fishType = (int)data[0][0];
    setTextureID(texID);
    loadEllipse(data,datasize);
    loadBehavior(data,datasize);
}

void CFishPoly::draw()
{
    //if (doAnimate) animate();

    glPushMatrix();

    /** This draws the bounding box
    VERTEX p[8];
    getBoundingBox(m_vOrigin, m_vVelocity, p);

    glBegin(GL_LINE_LOOP);
    for (int i = 0; i<8; i++) {
       glVertex3f(p[i].x, p[i].y, p[i].z);
    }
    glEnd();
    **/
    /** This draws the next bounding box
    getBoundingBox(getNextPosition(), m_vNextVelocity, p);

    glBegin(GL_LINE_LOOP);
    for (int i = 0; i<8; i++) {
       glVertex3f(p[i].x, p[i].y, p[i].z);
    }
    glEnd();
    **/

    if (m_schools && drawSchoolLines) {
       glDisable(GL_TEXTURE_2D);
       glDisable(GL_LIGHTING);
       glBegin(GL_LINES);
       if (calc_distance(m_vOrigin, m_vSchoolCenter) > m_fMaxSchoolRange) glColor3f(1.0,0.0,0.0);
       else glColor3f(1.0,1.0,0.0);
       glVertex3f(m_vOrigin.x,m_vOrigin.y,m_vOrigin.z);
       glVertex3f(m_vSchoolCenter.x,m_vSchoolCenter.y,m_vSchoolCenter.z);
       glEnd();
       glTranslatef(m_vOrigin.x,m_vOrigin.y,m_vOrigin.z);
       glBegin(GL_LINES);
       glColor3f(0.0,1.0,0.0);
       glVertex3f(0,0,0);
       glVertex3f(m_vSchoolVelocity.x*50,m_vSchoolVelocity.y*50,m_vSchoolVelocity.z*50);
       glEnd();
       glEnable(GL_LIGHTING);
       glEnable(GL_TEXTURE_2D);
    }
    else {
       glTranslatef(m_vOrigin.x,m_vOrigin.y,m_vOrigin.z);
    }

    /** This draws a line in the direction of velocity
    glBegin(GL_LINES);
    glVertex3f(0,0,0);
    glVertex3f(30*m_vVelocity.x,30*m_vVelocity.y,30*m_vVelocity.z);
    glEnd();
    **/
    /** This draws a line to the next position
    VERTEX v1 = getNextPosition();
    glBegin(GL_LINES);
    glVertex3f(0.0,0.0,0.0);
    glVertex3f(v1.x-m_vOrigin.x,v1.y-m_vOrigin.y,v1.z-m_vOrigin.z);
    glEnd();
    **/

    /** This draws a red vertical line if it's going to hit a rock
    if (willhit) {
      glDisable(GL_LIGHTING);
      glBegin(GL_LINES);
      glColor3f(1,0,0);
      glVertex3f(0,0,0);
      glVertex3f(0,10,0);
      glEnd();
      glEnable(GL_LIGHTING);
    }
    **/

    glRotatef(m_vRotation.y,0.0,1.0,0.0);
    glRotatef(m_vRotation.z,0.0,0.0,1.0);



    CPolygon::draw();
    glPopMatrix();

}


void CFishPoly::move()
{
   computeVelocity();

   m_vRotation.x = 0.0;
   GLfloat divisor = m_vVelocity.x*m_vVelocity.x+
                     m_vVelocity.z*m_vVelocity.z;
   if (divisor != 0) {
      m_vRotation.y = acos((-m_vVelocity.x)/sqrt(divisor)) * 180 / PI;
      if (m_vVelocity.z < 0) m_vRotation.y = -m_vRotation.y;
   }
   else {
       m_vRotation.y = 0.0;
   }

   GLfloat divisor1 = m_vVelocity.x*m_vVelocity.x+
                     m_vVelocity.y*m_vVelocity.y +
                     m_vVelocity.z*m_vVelocity.z;

   GLfloat divisor2 = m_vVelocity.x*m_vVelocity.x+ m_vVelocity.z*m_vVelocity.z;
   if (divisor1 != 0 && divisor2 != 0) {
      m_vRotation.z = acos((m_vVelocity.x*m_vVelocity.x+m_vVelocity.z*m_vVelocity.z)/(sqrt(divisor1)*sqrt(divisor2))) * 180 / PI;
      if (m_vVelocity.y > 0) m_vRotation.z = -m_vRotation.z;
   }
   else {
      m_vRotation.z = 0.0;
   }

   m_vOrigin.x += m_vVelocity.x;
   m_vOrigin.y += m_vVelocity.y;
   m_vOrigin.z += m_vVelocity.z;

}


void CFishPoly::computeVelocity()
{
   int now = GetTickCount();
   GLfloat nextx = m_vOrigin.x + m_vNextVelocity.x * (float)m_iSteps;
   GLfloat nexty = m_vOrigin.y + m_vNextVelocity.y * (float)m_iSteps;
   GLfloat nextz = m_vOrigin.z + m_vNextVelocity.z * (float)m_iSteps;
   GLfloat dvx = 0.0, dvy = 0.0, dvz = 0.0;
   GLfloat maxx = TANK_LENGTH/2.0 - m_fMinRange/2;
   GLfloat minx = -TANK_LENGTH/2.0 + m_fMinRange/2;
   GLfloat maxy = TANK_WIDTH/2.0 - m_fMinRange/2;
   GLfloat miny = -TANK_WIDTH/2.0 + m_fMinRange/2;
   GLfloat maxz = TANK_HEIGHT/2.0 - m_fMinRange/2;
   GLfloat minz = -TANK_HEIGHT/2.0 + m_fMinRange/2;

   if (now < m_iLastMove + m_iTimeStep) {
       m_iSteps++;
       float step = ((float)now - (float)m_iLastMove)/(float)m_iTimeStep;

       m_vVelocity.x = m_vLastVelocity.x + (m_vNextVelocity.x - m_vLastVelocity.x)*step;
       m_vVelocity.y = m_vLastVelocity.y + (m_vNextVelocity.y - m_vLastVelocity.y)*step;
       m_vVelocity.z = m_vLastVelocity.z + (m_vNextVelocity.z - m_vLastVelocity.z)*step;

       // Need to keep the velocity from being really close to
       // zero; it causes problems for some reason.
       if (m_vVelocity.x < 0.001 && m_vVelocity.x > -0.001) m_vVelocity.x = 0.001f;
       if (m_vVelocity.y < 0.001 && m_vVelocity.y > -0.001) m_vVelocity.y = 0.001f;
       if (m_vVelocity.z < 0.001 && m_vVelocity.z > -0.001) m_vVelocity.z = 0.001f;

       m_iCurrentMove = now;
       return;
   }
   else {
       m_iLastMove = now;
   }

   float rnd, rndmax = RAND_MAX;

   m_vLastVelocity = m_vNextVelocity;
   m_vLastPosition = m_vOrigin;
   if (calc_length(m_vVelocity)==0.0 && m_iLastMove != 0) {
       rnd = rand();
       dvx = 2.0*m_fMaxSpeed  * (rnd/rndmax) - m_fMaxSpeed;
       rnd = rand();
       dvy = 2.0*m_fMaxSpeed  * (rnd/rndmax) - m_fMaxSpeed;
       rnd = rand();
       dvz = 2.0*m_fMaxSpeed  * (rnd/rndmax) - m_fMaxSpeed;
   }
   else if (m_schools) {
      school();
      rnd = rand();
      float speed = m_fMaxSpeed  * (rnd/rndmax);
      float d = calc_distance(m_vOrigin, m_vSchoolCenter);
      if (d > m_fMaxSchoolRange && (rnd/rndmax) > 0.1) {

        dvx = (m_vSchoolCenter.x-m_vOrigin.x);
        dvy = (m_vSchoolCenter.y-m_vOrigin.y);
        dvz = (m_vSchoolCenter.z-m_vOrigin.z);

        float vL2 = calc_length(dvx, dvy, dvz);

        if (vL2 != 0.0) {                     
           rnd = rand();
           // give them a slight chance to get farther
           // away from the school, otherwise, hightail it back
           if ((rnd/rndmax) > 0.05) {
              dvx = (dvx/vL2)*m_fMaxSpeed*0.8;
              dvy = (dvy/vL2)*m_fMaxSpeed*0.8;
              dvz = (dvz/vL2)*m_fMaxSpeed*0.8;
           }
           else {
              dvx = m_vLastVelocity.x;
              dvy = m_vLastVelocity.y;
              dvz = m_vLastVelocity.z;
           }
        }
      }
      // else swim in the average direction
      // but at your own speed
      else {
        float vL2 = calc_length(m_vSchoolVelocity);
        if (vL2 != 0.0) {
           rnd = rand();
           dvx = (m_vSchoolVelocity.x/vL2)*speed*(1.5-(rnd/rndmax));
           rnd = rand();
           dvy = (m_vSchoolVelocity.y/vL2)*speed*(1.5-(rnd/rndmax));
           rnd = rand();
           dvz = (m_vSchoolVelocity.z/vL2)*speed*(1.5-(rnd/rndmax));
          //m_vNextVelocity = svel;
        }
      }
   }
   else {
       rnd = rand();
       dvx = m_vVelocity.x * (float)m_iSteps;
       if ((rnd/rndmax) * 10.0 < m_fActivity) {
           rnd = rand();
           dvx = 2.0*m_fMaxSpeed * (rnd/rndmax) - m_fMaxSpeed;
       }


       rnd = rand();
       dvy = m_vVelocity.y * (float)m_iSteps;
       if ((rnd/rndmax) * 10.0 < m_fActivity) {
           rnd = rand();
           dvy = 2.0*m_fMaxSpeed  * (rnd/rndmax) - m_fMaxSpeed;
       }

       rnd = rand();
       dvz = m_vVelocity.z * (float)m_iSteps;
       if ((rnd/rndmax) * 10.0 < m_fActivity) {
           rnd = rand();
           dvz = 0.5*2.0*m_fMaxSpeed  * (rnd/rndmax) - m_fMaxSpeed;
       }

   }

   
   if (dvx < 0.001 && dvx > -0.001) dvx = 0.001f;
   if (dvy < 0.001 && dvy > -0.001) dvy = 0.001f;
   if (dvz < 0.001 && dvz > -0.001) dvz = 0.001f;

   m_vVelocity = m_vLastVelocity;

   if (m_iSteps == 0) m_iSteps = 50; // averagish value

   m_vNextVelocity.x = dvx/m_iSteps;
   m_vNextVelocity.y = dvy/m_iSteps;
   m_vNextVelocity.z = dvz/m_iSteps;

   /*
   // limit the rotation angles
   // around y axis
   float divisor = sqrt(m_vLastVelocity.x*m_vLastVelocity.x+m_vLastVelocity.z*m_vLastVelocity.z)*
             sqrt(m_vNextVelocity.x*m_vNextVelocity.x+m_vNextVelocity.z*m_vNextVelocity.z);
   if (divisor != 0.0) {
       float thetay = acos((m_vLastVelocity.x*m_vNextVelocity.x+m_vLastVelocity.z*m_vNextVelocity.z)/divisor);
       if (thetay*180/PI > 60) {
            m_vNextVelocity.x = m_vLastVelocity.x*cos(PI*60/180) - m_vLastVelocity.z*sin(PI*60/180);
            m_vNextVelocity.z = m_vLastVelocity.x*sin(PI*60/180) + m_vLastVelocity.z*cos(PI*60/180);
       }
   }

   // z axis
   divisor = sqrt(m_vLastVelocity.x*m_vLastVelocity.x+m_vLastVelocity.y*m_vLastVelocity.y)*
             sqrt(m_vNextVelocity.x*m_vNextVelocity.x+m_vNextVelocity.y*m_vNextVelocity.y);
   if (divisor != 0.0) {
       float thetaz = acos((m_vLastVelocity.x*m_vNextVelocity.x+m_vLastVelocity.y*m_vNextVelocity.y)/divisor);
       if (thetaz*180/PI > 60) {
            m_vNextVelocity.x = m_vLastVelocity.x*cos(PI*60/180) - m_vLastVelocity.y*sin(PI*60/180);
            m_vNextVelocity.z = m_vLastVelocity.x*sin(PI*60/180) + m_vLastVelocity.y*cos(PI*60/180);
       }
   }
    */
   detectCollisions();

   float speed = calc_length(m_vNextVelocity);
   VERTEX newOrigin;
   rnd = rand();
   newOrigin.x =  0.5*((maxx-minx)  * (rnd/rndmax) - maxx);
   rnd = rand();
   newOrigin.y =  0.5*((maxy-miny)  * (rnd/rndmax) - maxy);
   rnd = rand();
   newOrigin.z =  0.5*((maxz-minz)  * (rnd/rndmax) - maxz);
   float distOrigin = calc_length(newOrigin);

   if ((m_vOrigin.x+2*m_vNextVelocity.x*m_iSteps) > maxx ||
       (m_vOrigin.x+2*m_vNextVelocity.x*m_iSteps) < minx) {
       m_vNextVelocity.x = -m_vNextVelocity.x*0.5;
      // if (m_schools) if (distOrigin != 0.0) m_vNextVelocity.x = -(newOrigin.x)*(speed/distOrigin);
   }
   if ((m_vOrigin.y+2*m_vNextVelocity.y*m_iSteps) > maxy ||
       (m_vOrigin.y+2*m_vNextVelocity.y*m_iSteps) < miny) {
       m_vNextVelocity.y = -m_vNextVelocity.y*0.5;
      // if (m_schools) if (distOrigin != 0.0) m_vNextVelocity.y = -newOrigin.y*(speed/distOrigin);
   }
   if ((m_vOrigin.z+2*m_vNextVelocity.z*m_iSteps) > maxz ||
       (m_vOrigin.z+2*m_vNextVelocity.z*m_iSteps) < minz) {
      m_vNextVelocity.z = -m_vNextVelocity.z*0.5;
     // if (m_schools) if (distOrigin != 0.0)  m_vNextVelocity.z = -newOrigin.z*(speed/distOrigin);
   }

   m_iTotalSteps = m_iSteps;
   m_iSteps = 0;

}

void CFishPoly::animate() {
    int n = getVertexCount();
    VERTEX v;
    if (m_iTotalSteps == 0) m_iTotalSteps = 1;
    float x=0.0,y=0.0, tailx=0.0 , t = ((float)m_iSteps/(float)m_iTotalSteps) * 2 * PI ;
    for (int i = 0; i < n; i++) {
        v = getOriginalVertex(i);
        x = (v.x-getMinX())/(getMaxX()-getMinX()) * 0.5*PI ;
        y = (v.y-getMinY())/(getMaxY()-getMinY()) * 4 * PI ;
        v.z += (getMaxX()/30)*sin(x + t);
        if ((v.x-getMinX()) > 0.85 * (getMaxX()-getMinX())) {
            tailx = (getMaxX()-getMinX()) - (v.x-getMinX());
            tailx = (0.15 * (getMaxX()-getMinX()) - tailx)/(0.15 * (getMaxX()-getMinX()));
            v.z += (getMaxX()/60)*tailx*sin(y + t);
        }
        setVertex(i, v.x, v.y, v.z);
    }
    renormalize();
}

void CFishPoly::loadBehavior(float data[][7], int datasize) {
    m_fActivity = data[2][0];
    m_fMaxSpeed = data[2][1];
    m_fMinRange = data[2][2];
    m_fMinFishRange = data[2][3];
    if ((int)data[2][4] == 1) m_schools = TRUE;
    else m_schools = FALSE;
    m_fMaxSchoolRange = data[2][5];
    m_Rank = data[2][6];
}

void CFishPoly::loadBehavior(char* fName) {
    std::ifstream ld(fName);
    char ch[256], *pch;
    int n;
    float vals[15];

    int count = 0;
    while (ld.getline(ch,256)) {
        n = 0;

        if (count == 2) {
           pch = strtok (ch, ",");
           while (pch != NULL)
           {
               vals[n] = atof(pch);
               pch = strtok (NULL, ",");

               n++;
           }
           m_fActivity = vals[0];
           m_fMaxSpeed = vals[1];
           m_fMinRange = vals[2];
           m_fMinFishRange = vals[3];
           if (vals[4] == 1) m_schools = TRUE;
           else m_schools = FALSE;
           m_fMaxSchoolRange = vals[5];
           m_Rank = vals[6];
        }
        count++;
    }
    //outfile << m_fActivity << " " << m_fMaxSpeed << "\n";
}

void CFishPoly::setAllFish(CFishPoly * fishes, int fishCount)  {
    m_fish = fishes;
    m_fishCount = fishCount;
}

void CFishPoly::setAllRocks(CRockPoly * rocks, int rockCount)  {
    m_rocks = rocks;
    m_rockCount = rockCount;
}

void CFishPoly::detectCollisions() {
    VERTEX pos = getNextPosition();
    VERTEX curpos = getPosition();
    VERTEX fpos;
    float d;
	int i;
    for (i = 0; i < m_fishCount; i++) {
        if (this != &m_fish[i]) {
            fpos = m_fish[i].getNextPosition();
            d = calc_distance(pos, fpos);
            if (d < m_fMinFishRange) {
               if (m_fish[i].getRank() == m_Rank) {
                  //m_vNextVelocity.x = -0.5*m_vNextVelocity.x;
                  //m_vNextVelocity.y = -0.5*m_vNextVelocity.y;
                  //m_vNextVelocity.z = -0.5*m_vNextVelocity.z;
               }
               else if (m_fish[i].getRank() > m_Rank) {
                  m_vNextVelocity.x = -m_vNextVelocity.x;
                  m_vNextVelocity.y = -m_vNextVelocity.y;
                  m_vNextVelocity.z = -m_vNextVelocity.z;
               }
            }
        }
    }
    VERTEX curbox[8], nextbox[8];
    float deg = 0.0;
    getBoundingBox(curpos, m_vVelocity, curbox);

    willhit=FALSE;
    for (i = 0; i < m_rockCount; i++) {
         getBoundingBox(getNextPosition(), m_vNextVelocity, nextbox);
        // while (hitsRock(&m_rocks[i]) > 0 && deg < 180.0) {

            /*
            deg += 20.0;
            VERTEX newVel;
            newVel.x = m_vNextVelocity.x*cos(PI*deg/180) - m_vNextVelocity.z*sin(PI*deg/180);
            newVel.z = m_vNextVelocity.x*sin(PI*deg/180) + m_vNextVelocity.z*cos(PI*deg/180);

            m_vNextVelocity.x = newVel.x;
            m_vNextVelocity.z = newVel.z;
            */
        if (hitsRock(&m_rocks[i]) > 0) {
            willhit=TRUE;
            VERTEX rockpos = m_rocks[i].getPosition();
            VERTEX newVel;
            float curSpeed = calc_length(m_vNextVelocity);
            newVel.x = curpos.x - rockpos.x;
            newVel.y = curpos.y - rockpos.y;
            newVel.z = curpos.z - rockpos.z;
            float newSpeed = calc_length(newVel);
            if (newSpeed != 0) {
                m_vNextVelocity.x = newVel.x/newSpeed*curSpeed;
                m_vNextVelocity.y = newVel.y/newSpeed*curSpeed;
                m_vNextVelocity.z = newVel.z/newSpeed*curSpeed;
            }

        //    getBoundingBox(getNextPosition(), m_vNextVelocity, nextbox);
        }
    }
}

void CFishPoly::school() {
    if (m_schooled) return;

    VERTEX pos = getPosition();
    VERTEX fpos, spos, fvel, svel;
    int n = 0, m = 0;
    int type = getType();
    spos.x = 0.0; spos.y = 0.0; spos.z = 0.0;
    svel.x = 0.0; svel.y = 0.0; svel.z = 0.0;
    fpos.x = 0.0; fpos.y = 0.0; fpos.z = 0.0;
    fvel.x = 0.0; fvel.y = 0.0; fvel.z = 0.0;

    for (int i = 0; i < m_fishCount; i++) {
        if (type == m_fish[i].getType()) {
            fpos = m_fish[i].getPosition();
            fvel = m_fish[i].getVelocity();
            spos.x += fpos.x;
            spos.y += fpos.y;
            spos.z += fpos.z;
            if (calc_distance(fpos, m_vSchoolCenter) <= m_fMaxSchoolRange) {
               svel.x += fvel.x;
               svel.y += fvel.y;
               svel.z += fvel.z;
               //outfile <<"--- " << i << ". " << fvel.x << " " << fvel.y << " " << fvel.z << "\n";
               m++;
            }

            n++;
        }
    }
    if (n > 0) {
       spos.x /= (float) n;
       spos.y /= (float) n;
       spos.z /= (float) n;
    }
   // outfile << m << ". " << svel.x << " " << svel.y << " " << svel.z << "\n";
    if (m > 0) {
       svel.x /= (float) m;
       svel.y /= (float) m;
       svel.z /= (float) m;
    }

    //outfile << m << ". " << svel.x << " " << svel.y << " " << svel.z << "\n";

    for (i = 0; i < m_fishCount; i++) {
        if (type == m_fish[i].getType()) {
            m_fish[i].setSchoolVelocity(svel);
            m_fish[i].setSchoolCenter(spos);
            m_fish[i].setSchooled(TRUE);

        }
    }
    setSchooled(FALSE);
}

VERTEX CFishPoly::getNextPosition() {
    VERTEX v;
     v.x = m_vLastPosition.x+(m_vNextVelocity.x + m_vLastVelocity.x)*m_iTotalSteps/2;
     v.y = m_vLastPosition.y+(m_vNextVelocity.y + m_vLastVelocity.y)*m_iTotalSteps/2;
     v.z = m_vLastPosition.z+(m_vNextVelocity.z + m_vLastVelocity.z)*m_iTotalSteps/2;
    return v;
}

VERTEX CFishPoly::getPosition() {
    return m_vOrigin;
}

VERTEX CFishPoly::getNextVelocity() {
    return m_vNextVelocity;
}

VERTEX CFishPoly::getVelocity() {
    return m_vVelocity;
}

int CFishPoly::getType() {
    return m_fishType;
}

float CFishPoly::getRank() {
    return m_Rank;
}

void CFishPoly::setSchooled(bool b) {
    m_schooled = b;
}

void CFishPoly::setSchoolVelocity(VERTEX v) {
    m_vSchoolVelocity = v;
}

void CFishPoly::setSchoolCenter(VERTEX v) {
    m_vSchoolCenter = v;
}

void CFishPoly::getBoundingBox(VERTEX pos, VERTEX vel, VERTEX* points) {
    points[0].x = getMinX();
    points[0].y = getMaxY();
    points[0].z = getMaxZ();
    points[1].x = getMinX();
    points[1].y = getMaxY();
    points[1].z = getMinZ();
    points[2].x = getMaxX();
    points[2].y = getMaxY();
    points[2].z = getMinZ();
    points[3].x = getMaxX();
    points[3].y = getMaxY();
    points[3].z = getMaxZ();
    points[4].x = getMinX();
    points[4].y = getMinY();
    points[4].z = getMaxZ();
    points[5].x = getMinX();
    points[5].y = getMinY();
    points[5].z = getMinZ();
    points[6].x = getMaxX();
    points[6].y = getMinY();
    points[6].z = getMinZ();
    points[7].x = getMaxX();
    points[7].y = getMinY();
    points[7].z = getMaxZ();

    // now rotate them int the order of the velocity
    float rotx = 0.0, roty = 0.0, rotz = 0.0;
    GLfloat divisor = vel.x*vel.x+vel.z*vel.z;
    if (divisor != 0) {
       roty = acos((-vel.x)/sqrt(divisor));
       if (vel.z*vel.x < 0) roty = -roty;
    }
    else {
        roty= 0.0;
    }

    GLfloat divisor1 = vel.x*vel.x+vel.y*vel.y +vel.z*vel.z;
    GLfloat divisor2 = vel.x*vel.x+ vel.z*vel.z;
    if (divisor1 != 0 && divisor2 != 0) {
       rotz = acos((vel.x*vel.x+vel.z*vel.z)/(sqrt(divisor1)*sqrt(divisor2)));
       if (vel.y > 0) rotz = -rotz;
    }
    else {
       rotz = 0.0;
    }
    float tx,ty,tz;
    for (int i =0; i < 8; i++) {
        tx = points[i].x * cos(rotz) - points[i].y * sin(rotz);
        ty = points[i].y * cos(rotz) + points[i].x * sin(rotz);

        points[i].x = tx;
        points[i].y = ty;

        tx = points[i].x * cos(roty) - points[i].z * sin(roty);
        tz = points[i].z * cos(roty) + points[i].x * sin(roty);

        points[i].x = tx;
        points[i].z = tz;

        points[i].x += pos.x;
        points[i].y += pos.y;
        points[i].z += pos.z;
    }
}

int CFishPoly::hitsRock(CRockPoly * rock) {
    VERTEX curbox[8], nextbox[8];
    getBoundingBox(getPosition(), m_vVelocity, curbox);

    int hitcount = 0;
    for (int j = 0; j < 8; j++) {
        getBoundingBox(getNextPosition(), m_vNextVelocity, nextbox);
        if (rock[0].isPathIn(curbox[j], nextbox[j], m_fMinRange)) {
           hitcount++;
        }
    }

    return hitcount;
}












